class Bulletin
  include Mongoid::Document
  include Mongoid::Timestamps

  include OrbitModel::Status
  include OrbitModel::Impression
  # encoding: utf-8
  include OrbitTag::Taggable
  include OrbitCategory::Categorizable
  include Slug
  require 'bulletin_model/cache'
  include ::BulletinModel::Cache
  attr_accessor :org_tag_ids
  def tags=(ids)
    self.org_tag_ids = self.tag_ids
    super(ids)
  end
  def []=(index,value)
    if index.to_s=='tags'
      self.org_tag_ids = self.tag_ids
    end
    super(index,value)
  end
  SubPart.class_eval { include BulletinModel::Cache }
  Page.class_eval { include BulletinModel::Cache }
  before_destroy do
    AnnsCache.all.destroy
  end
  field :is_edit, type: Boolean, default: false #use to check whether the preview record changed
  field :copy_id
  field :custom_carousel_image_type, :type => Integer, :default => 0 # 0: default, 1: carousel, 2: album
  field :custom_carousel_image_width, type: String, default: ""
  field :image_display_class, type: String, default: "full-size-img" #3 choices: full-size-img , pull-left , pull-right
  field :add_to_calendar,type: Boolean,default: false
  field :calendar_start_date, :type => DateTime
  field :calendar_end_date, :type => DateTime
  field :calendar_all_day,type: Boolean,default: false
  field :calendar_type_id
  field :event_id
  field :page_id
  field :title, as: :slug_title, type: String, localize: true
  field :subtitle, localize: true
  field :text, localize: true
  field :create_user_id
  field :update_user_id
  field :public, :type => Boolean, :default => true
  field :postdate , :type => DateTime, :default => Time.now
  field :deadline , :type => DateTime
  field :rss2_sn
  field :approved, :type => Boolean, :default => false
  field :is_preview, :type => Boolean, :default => false
  field :expirable_created_at, type: DateTime
  field :rejected, :type => Boolean, :default => false
  field :reapproval, :type => Boolean, :default => false
  field :rejection_reason
  field :is_external_link, :type => Boolean, :default => false
  field :external_link
  field :display_subtitle, :type => Boolean, :default => false
  field :display_img, :type => Boolean, :default => false

  field :email_id
  field :email_sent, :type => Boolean, :default => false
  field :email_sentdate , :type => DateTime
  field :email_member_ids
  field :other_mailaddress
  field :image_description, localize: true
  field :top_end_date, :type => DateTime
  field :open_comment, :type => Boolean, :default => false
  field :comment_end_time, :type => DateTime
  field :comment_role, :type => Array, :default => []
  field :enable_sub_annc, :type => Boolean, :default => false
  field :sub_annc_list, :type => Array, :default => []
  field :custom_sub_annc_title_trans, :type => String, :default => "", :localize => true
  field :display_sub_annc_date, :type => Boolean, :default => false
  mount_uploader :image, ImageUploader

  has_many :bulletin_links, :autosave => true, :dependent => :destroy
  has_many :bulletin_files, :autosave => true, :dependent => :destroy
  has_many :bulletin_comments, :autosave => true, :dependent => :destroy
  has_many :bulletin_carousel_images, :autosave => true, :dependent => :destroy
  accepts_nested_attributes_for :bulletin_files, :allow_destroy => true
  accepts_nested_attributes_for :bulletin_links, :allow_destroy => true
  accepts_nested_attributes_for :bulletin_carousel_images, :allow_destroy => true

  before_destroy :destroy_email

  scope :open_in_future, ->{where(:is_hidden.ne=>true,:is_preview.ne => true,:postdate.gt=>Time.now).order(postdate: :asc)}
  scope :can_display_and_sorted, ->{where(:is_hidden.ne=>true,:is_preview.ne => true).any_of({:postdate.lte=>Time.now, :deadline.gte=>Time.now},{:postdate.lte=>Time.now, :deadline=>nil},{:postdate=>nil}).order(is_top: :desc,postdate: :desc,id: :desc)}
  scope :is_approved, ->{where(:approved => true)}
  scope :is_approved_and_show, ->{where(:approved => true,:is_hidden.ne=>true,:is_preview.ne => true)}
  scope :filter_cats_and_tags, ->(cats,tags) {filter_by_widget_categories(cats,false).filter_by_tags(tags)}
  before_create :set_expire
  before_save :check_limit
  index({postdate: 1}, { unique: false, background: true })
  index({is_top: -1,postdate: -1, _id: -1}, { unique: false, background: true })
  def get_org_model
    if self.is_preview
      org_model = nil
      if self.copy_id
        org_model = self.class.find(self.copy_id) rescue nil
      else
        org_model = self.class.where(:title=>self.title,:is_preview.ne=>true).desc(:updated_at).first
      end
      org_model.nil? ? self : org_model
    else
      self
    end
  end
  def to_calendar_param
    self.to_param
  end
  def calendar_type
    CalendarType.where(:category_id.in => self.calendar_type_id)
  end
  def event
    if !self.event_id.nil?
      Event.where(:id => self.event_id).first
    else
      nil
    end
  end
  def check_limit
    check_status_limit(update_user)
  end
  def check_status_limit(user,check_only=false)
    role_ids = user.member_profile.roles.map(&:id) rescue []
    status_settings = (role_ids.collect do |role_id|
      AnnouncementSetting.first.anns_status_settings.select{|v| v.role_id.to_s == role_id.to_s}
    end.flatten rescue [])
    reach_limit = []
    if status_settings.count != 0
      reach_limit = status_settings.collect do |status_setting|
        status = status_setting.status
        if status_setting.top_limit.to_i <= self.class.where(:is_preview.ne=>true,:update_user_id.in => Role.find(status_setting.role_id).member_profiles.collect(&:user).flatten.uniq.map{|v| v.id},status => true).count
          if !check_only
            if self[status] && !self.class.where(id:self.id).first[status]
              self[status] = false
              nil
            end
          else
            status
          end
        else
          nil
        end
      end.compact
      reach_limit = reach_limit.group_by{|v| v}.collect do |k,v|
                      if v.count >= user.member_profile.roles.count
                        k
                      else
                        nil
                      end
                    end.compact
    end
    reach_limit
  end
  def set_expire
    self.expirable_created_at = Time.now if self.is_preview
    return true
  end

  def update_user
    User.find(update_user_id) rescue nil
  end

  def update_user=(user)
    self.update_user_id = user.id
  end

  def email_members
    MemberProfile.find(self.email_member_ids) rescue []
  end

  def email_addresses
    addresses = self.email_members.collect{|member| member.email} rescue []
    addresses = addresses +[self.other_mailaddress] if !self.other_mailaddress.blank?
    addresses.flatten
  end

  def email
    mail = Email.find(self.email_id) rescue nil
  end

  def expired?
    (self.deadline < Time.now) rescue false
  end

  def destroy_email
    mail = Email.find(self.email_id) rescue nil
    mail.destroy if !mail.nil?
  end

  def self.remove_expired_status
    self.where(:is_top => true, :top_end_date.ne => nil, :top_end_date.lt => Time.now).each do |b|
      b.is_top = false
      b.top_end_date = nil
      b.save
    end
  end

  def display_subtitle?
    self.display_subtitle rescue false
  end

  def display_img?
    self.display_img rescue false
  end
  def comments
    self.bulletin_comments.select{|v| !v.is_hidden}
  end
  def open_comment_for_user(user)
    role_ids = user.member_profile.roles.collect{|v| v.id.to_s} rescue ['visitor']
    self.open_comment && (self.comment_end_time.blank? || self.comment_end_time > Time.now) && (self.comment_role.any?{|v| role_ids.include?(v)} || self.comment_role.include?('visitor') || (self.comment_role.include?('all_member') && role_ids[0] != 'visitor'))
  end
  def statuses
        statuses = []
        statuses << top_text if is_top?
        statuses << hot_text if is_hot?
        statuses << hidden_text if is_hidden?
        statuses
      end
  def statuses_with_classname
    statuses = []
    statuses << {"name" => top_text, "classname" => "top"} if is_top?
    statuses << {"name" => hot_text, "classname" => "hot"} if is_hot?
    statuses << {"name" => hidden_text, "classname" => "hidden"} if is_hidden?
    statuses
  end
  def status_for_table
    status = ""
    status << "<span class='label label-success'>#{top_text}</span> " if self.is_top
    status << "<span class='label label-important'>#{hot_text}</span> " if self.is_hot
    status << "<span class='label'>#{hidden_text}</span>"if self.is_hidden
    status.html_safe
  end
  def top_text
    I18n.t("announcement.status.top")
  end
  def hot_text
    I18n.t("announcement.status.hot")
  end
  def hidden_text
    I18n.t("announcement.status.hidden")
  end
  def carousel_image_type
    (self.custom_carousel_image_type == 0 ? AnnouncementSetting.last.carousel_image_type : self.custom_carousel_image_type - 1) rescue 0
  end
  def carousel_image_width
    (self.custom_carousel_image_width.blank? ? AnnouncementSetting.last.carousel_image_width : self.custom_carousel_image_width) rescue "75%"
  end
  def self.agenda_events(agenda_start, agenda_end,read_more_url)
    events = self.monthly_event(agenda_start, agenda_end).convert_front(read_more_url)
  end
  def self.monthly_event(start_date,end_date)
    self.any_of({:postdate.lte => start_date,:deadline.gte => start_date},{:postdate.gte => start_date,:deadline.lte => end_date},{:postdate.lte => end_date,:deadline.gte => end_date}).asc(:postdate)
  end
  def self.convert_front(read_more_url)
    self.all.collect do |re|
      {:id => re.id.to_s,
       :title=>re.title,
       :note=>re.subtitle || "",
       :allDay => false,
       :color => nil,
       :url_linked => (re.is_external_link ? re.external_link : "#{read_more_url}/#{re.to_param}" rescue ""),
       :start => re.postdate,
       :end => re.deadline}
    end
  end
  def get_sub_annc_title_trans(locale=I18n.locale)
    I18n.with_locale(locale) do
      self.custom_sub_annc_title_trans.blank? ? (AnnouncementSetting.first.get_sub_annc_title_trans(locale) rescue I18n.t("announcement.table.title")) : self.custom_sub_annc_title_trans
    end
  end
  def get_sub_annc_list
    sub_anncs = self.class.where(:id.in=>self.sub_annc_list).to_a
    sub_anncs = sub_anncs.sort_by{|a| sub_annc_list.index(a.id.to_s)}
  end
  def display_postdate
    self.postdate.present? ? self.postdate.strftime("%Y/%m/%d") : ""
  end
end
